#include <cstdint>
#include <iostream>
using namespace std;

#include "cfunctions.h"
#include "cppfunctions.h"
class LCD
{
public:
    void DisplayString(const uint8_t *text);
    void DisplayString(const char *text);
};

static const uint8_t *u8_text = (uint8_t*)"C++ uint8_t text";

// In C++, const is not global unless extern is specified
extern const int my_global = 2; 

int main()
{
    LCD oLcd;
    
    // Call C function from C++ 
    // LCD_DisplayString needs C linkage
    // A cast is required
    LCD_DisplayString((uint8_t*)"To avoid this uint8_t cast, use C++ wrappers");

    // Use C++ wrappers instead
    oLcd.DisplayString("C++ char text");
    oLcd.DisplayString(u8_text);
    
    c_function(1,2);
    print_my_global();
    
    return 0;
}

void LCD::DisplayString(const uint8_t *text)
{
    cout << "[uint8_t]:: ";
    LCD_DisplayString(text); // Calls the C function (C linkage needed)
}

void LCD::DisplayString(const char *text)
{
    cout << "[char]:: ";
    DisplayString((uint8_t*) text); // The cast is done here
}

void cpp_function(int a, int b)
{
    // This C++ function needs C linkage to be visible to C applications
    cout << "[cpp_function]: Hello from C++" << endl;
}
